@@ -472,6 +472,35 @@ def lattice_coords_transformer(
     return new_cart_coords
 
 
+# https://www2.clarku.edu/faculty/djoyce/wallpaper/seventeen.html
+def get_2d_lattice(atoms={}, round_digits=2):
+    """Get 2D lattice type."""
+    lattices = {
+        "hexagonal": 0,
+        "square": 1,
+        "rectangle": 2,
+        "rhombus": 3,
+        "parallelogram": 4,
+    }
+    abc = atoms["abc"]
+    a = round(abc[0], round_digits)
+    b = round(abc[1], round_digits)
+    angles = atoms["angles"]
+    beta = round(angles[2], round_digits)
+    if a == b and (beta == 120 or beta == 60):
+        lat = "hexagonal"
+    elif a == b and beta == 90:
+        lat = "square"
+    elif a != b and beta == 90:
+        lat = "rectangle"
+    elif a == b and beta != 90:
+        lat = "rhombus"  # centered rectangle
+    else:
+        lat = "parallelogram"  # oblique
+    lat_type = lattices[lat]
+    return [lat, lat_type]
+
+
 """
 if __name__ == "__main__":
 
